#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <error.h>
#include <wait.h>
#include <unistd.h>

const int rd_sz = 8000;

inline void child_process(int pipefd, int child_num) {
  char inpipe[8192];
  
  lockf(pipefd, F_LOCK, 0);  // 互斥
  read(pipefd, inpipe, rd_sz);
  printf("Child process %d (PID: %d) received message: %s", child_num, getpid(), inpipe);
  lockf(pipefd, F_ULOCK, 0);
  sleep(1);
  exit(0);
}

int main() {
  int pid1, pid2, pid3;
  int fd[2];
  char outpipe[8192];
  pipe(fd);  // 创建一个管道
  perror("pipe");//返回前面一个函数的执行系统错误信息

  pid1 = fork();
  if (pid1 == -1) {
    perror("fork");
    exit(1);
  } else if (pid1 == 0) {
    child_process(fd[0], 1);
  }

  pid2 = fork();
  if (pid2 == -1) {
    perror("fork");
    exit(1);
  } else if (pid2 == 0) {
    // 子进程p2
    child_process(fd[0], 2);
  }

  pid3 = fork();
  if (pid3 == -1) {
    perror("fork");
    exit(1);
  } else if (pid3 == 0) {
    // 子进程p3
    child_process(fd[0], 3);
  }

  // 父进程
  sprintf(outpipe, "This is a message from the parent process! (PID: %d)\n", getpid());
  write(fd[1], outpipe, rd_sz);
  write(fd[1], outpipe, rd_sz);
  write(fd[1], outpipe, rd_sz);
  
  // 等待子进程p1结束  
  waitpid(pid1, NULL, 0);
  // 等待子进程p2结束
  waitpid(pid2, NULL, 0);
  // 等待子进程p3结束
  waitpid(pid3, NULL, 0);

  close(fd[0]);
  close(fd[1]);
  return 0;
}